@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _PHTTPFORM_H
-#define _PHTTPFORM_H
+#ifndef PTLIB_HTTPFORM_H
+#define PTLIB_HTTPFORM_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -896,7 +896,7 @@ class PHTTPConfigSectionList : public PHTTPString
 
 #endif // P_HTTPFORMS
 
-#endif // _HTTPFORM_H
+#endif // PTLIB_HTTPFORM_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
