@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21486 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-11-09 23:38:52 +0000 (Sun, 09 Nov 2008) $
  */
 
 #include "precompile.h"
@@ -40,7 +40,7 @@ PCREATE_PROCESS(IPV6Test);
 
 
 IPV6Test::IPV6Test()
-  : PProcess("Post Increment", "dtmftest", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER)
+  : PProcess("Post Increment", "ipv6test", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER)
 {
 }
 
@@ -228,12 +228,12 @@ void IPV6Test::Main()
       PIPSocket::InterfaceEntry if_entry = if_table[i];
       cout << i << " " << if_entry << endl;
     }
-    cout << "manual check";
+    cout << "Please do manual check ...";
     cout << endl;
   }
   {
     // test #9 - see if URLs decode correctly
-    cout << "test #9: check if parsing IPV6 URLs works" << endl;
+    cout << "test #9: Please do manual check if parsing IPV6 URLs works" << endl;
 
     PURL url("h323:@[::ffff:220.244.81.10]:1234");
     PString addrStr = url.GetHostName();
