@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
- * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Revision: 21977 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-01 23:43:21 +0000 (Sun, 01 Feb 2009) $
  */
 
 #ifdef __GNUC__
@@ -61,6 +61,26 @@ class PServiceMacros_list : public PServiceMacros_base
 static const PTime ImmediateExpiryTime(0, 0, 0, 1, 1, 1980);
 
 
+/////////////////////////////////////////////////////////////////////////////
+
+PSystemLog::PSystemLog(Level level)   ///< only messages at this level or higher will be logged
+  : iostream(cout.rdbuf()) 
+{ 
+  logLevel = level; 
+  buffer.log = this; 
+  init(&buffer); 
+}
+
+PSystemLog::PSystemLog(const PSystemLog & other)
+  : PObject(other), iostream(cout.rdbuf()) 
+{
+}
+
+PSystemLog & PSystemLog::operator=(const PSystemLog &)
+{ 
+  return *this; 
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PHTTPServiceProcess::PHTTPServiceProcess(const Info & inf)
