@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21579 $
- * $Author: csoutheren $
- * $Date: 2008-11-20 05:31:36 +0000 (Thu, 20 Nov 2008) $
+ * $Revision: 22156 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-04 06:28:29 +0000 (Wed, 04 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -40,38 +40,72 @@
 #include <ptlib/pfactory.h>
 #include <ptlib/pprocess.h>
 #include <ptlib/svcproc.h>
+#include "../../../version.h"
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-namespace PWLibStupidLinkerHacks {
-  extern int loadVideoForWindowsStuff;
-};
-#endif
 #endif
 
-#if defined(P_MINGW)
 namespace PWLibStupidLinkerHacks {
-  extern int loadFakeVideoStuff;
-#if defined(P_AUDIO)
-  extern int loadWindowsMultimediaStuff;
-#endif
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-  extern int loadVideoForWindowsStuff;
-#endif
-#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
-  extern int loadDirectShowStuff;
-#endif
+
+
+#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+#  ifdef P_VIDEO
+     extern int loadFakeVideoStuff;
+#    if defined(P_MINGW) || defined(_WIN32)
+#      ifdef P_VFW_CAPTURE
+         extern int loadVideoForWindowsStuff;
+#      endif
+#      if defined(P_DIRECTSHOW)
+         extern int loadDirectShowStuff;
+#      endif
+#    endif
+#    ifdef P_FFVDEV
+       extern int loadFFVideoFileStuff;
+#    endif
+#    ifdef P_VIDFILE
+       extern int loadVideoFileStuff;
+#    endif
+#  endif 
+
+#  ifdef P_AUDIO
+#    if defined(P_MINGW) || defined(_WIN32)
+       extern int loadWindowsMultimediaStuff;
+#    endif
+#    ifdef P_MACOSX
+       extern int loadCoreAudioStuff;
+#    endif
+#  endif
+
+#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+}; // namespace PWLibStupidLinkerHacks {
+
+
+static const char * const VersionStatus[PProcess::NumCodeStatuses] = { "alpha", "beta", "." };
+
+
+class PExternalThread : public PThread
+{
+  PCLASSINFO(PExternalThread, PThread);
+  public:
+    PExternalThread()
+    {
+      SetThreadName(PString::Empty());
+      PTRACE(5, "PTLib\tCreated external thread " << this << " for id " << GetCurrentThreadId());
+    }
+
+    ~PExternalThread()
+    {
+      PTRACE(5, "PTLib\tDestroyed external thread " << this << " for id " << GetThreadId());
+    }
+
+    void Main()
+    {
+    }
 };
-#endif
-     
-#ifdef P_MACOSX
-namespace PWLibStupidLinkerHacks {
-  extern int loadFakeVideoStuff;
-  extern int loadShmVideoStuff;
-  extern int loadCoreAudioStuff;
-};
-#endif
+
 
 class PSimpleThread : public PThread
 {
@@ -92,6 +126,9 @@ class PSimpleThread : public PThread
 };
 
 
+#define new PNEW
+
+
 #ifndef __NUCLEUS_PLUS__
 static ostream * PErrorStream = &cerr;
 #else
@@ -242,7 +279,7 @@ PTHREAD_MUTEX_RECURSIVE_NP
 
   void OpenTraceFile(const char * newFilename)
   {
-    if (newFilename != NULL)
+    if (newFilename != NULL && *newFilename != '\0')
       filename = newFilename;
 
     if (filename == NULL)
@@ -336,6 +373,7 @@ void PTrace::Initialise(unsigned level, const char * filename, const char * roll
                   << " by " << process.GetManufacturer()
                   << " on " << process.GetOSClass() << ' ' << process.GetOSName()
                   << " (" << process.GetOSVersion() << '-' << process.GetOSHardware()
+                  << ") with PTLib (v" << process.GetLibVersion()
                   << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
                   << End;
 #endif
@@ -531,14 +569,8 @@ ostream & PTrace::End(ostream & paramStream)
     // should work pretty universally.
     info.stream->width((threadInfo != NULL ? threadInfo->traceLevel : info.currentLevel) + 1);
   }
-  else {
-#ifdef _WIN32_WCE
-    if (!PIsDescendant(info.stream, PDebugStream) && info.stream != &cerr)
-      *info.stream << "\r\n";
-    else
-#endif
+  else
     *info.stream << '\n';
-  }
   info.stream->flush();
 
   info.Unlock();
@@ -616,11 +648,9 @@ PTrace::Block::~Block()
 void PTrace::Cleanup()
 {
 #if P_HAS_THREADLOCAL_STORAGE
-  {
-    PThreadLocalStorage<PThread::TraceInfo> & key = PTraceInfo::Instance().traceStorageKey;
-    delete key.Get();
-    key.Set(NULL);
-  }
+  PThreadLocalStorage<PThread::TraceInfo> & key = PTraceInfo::Instance().traceStorageKey;
+  delete key.Get();
+  key.Set(NULL);
 #endif
 }
 
@@ -972,6 +1002,8 @@ PArgList::PArgList(const char * theArgStr,
   // get the program arguments
   if (theArgStr != NULL)
     SetArgs(theArgStr);
+  else
+    SetArgs(PStringArray());
 
   // if we got an argument spec - so process them
   if (theArgumentSpec != NULL)
@@ -1098,6 +1130,7 @@ void PArgList::SetArgs(const PStringArray & theArgs)
   parameterIndex.SetSize(argumentArray.GetSize());
   for (PINDEX i = 0; i < argumentArray.GetSize(); i++)
     parameterIndex[i] = i;
+  m_argsParsed = 0;
 }
 
 
@@ -1111,8 +1144,8 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
 
   // If not in parse all mode, have been parsed before, and had some parameters
   // from last time, then start argument parsing somewhere along instead of start.
-  if (optionsBeforeParams && !optionLetters && parameterIndex.GetSize() > 0)
-    arg = parameterIndex[parameterIndex.GetSize()-1] + 1;
+  if (optionsBeforeParams && !optionLetters && m_argsParsed > 0)
+    arg = m_argsParsed;
 
   // Parse the option specification
   optionLetters = "";
@@ -1161,10 +1194,18 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
       parameterIndex.SetSize(param+1);
       parameterIndex[param++] = arg;
     }
-    else if (optionsBeforeParams && parameterIndex.GetSize() > 0)
+    else if (optionsBeforeParams && parameterIndex.GetSize() > 0) {
+      m_argsParsed = arg;
       break;
-    else if (argStr == "--") // ALL remaining args are parameters not options
-      hadMinusMinus = PTrue;
+    }
+    else if (argStr == "--") {
+      if (optionsBeforeParams)
+        hadMinusMinus = PTrue; // ALL remaining args are parameters not options
+      else {
+        m_argsParsed = arg+1;
+        break;
+      }
+    }
     else if (argStr[1] == '-')
       ParseOption(optionNames.GetValuesIndex(argStr.Mid(2)), 0, arg, canHaveOptionString);
     else {
@@ -1457,24 +1498,33 @@ PString PConfigArgs::CharToString(char ch) const
 // PProcess
 
 PProcess * PProcessInstance;
-int PProcess::p_argc;
-char ** PProcess::p_argv;
-char ** PProcess::p_envp;
 
 typedef std::map<PString, PProcessStartup *> PProcessStartupList;
 
-int PProcess::_main(void *)
+int PProcess::InternalMain(void *)
 {
   Main();
   return terminationValue;
 }
 
 
-void PProcess::PreInitialise(int c, char ** v, char ** e)
+void PProcess::PreInitialise(int c, char ** v, char **)
 {
-  p_argc = c;
-  p_argv = v;
-  p_envp = e;
+  if (executableFile.IsEmpty()) {
+    PString execFile = v[0];
+    if (PFile::Exists(execFile))
+      executableFile = execFile;
+    else {
+      execFile += ".exe";
+      if (PFile::Exists(execFile))
+        executableFile = execFile;
+    }
+  }
+
+  if (productName.IsEmpty())
+    productName = executableFile.GetTitle().ToLower();
+
+  arguments.SetArgs(c-1, v+1);
 }
 
 
@@ -1487,95 +1537,73 @@ static PProcessStartupList & GetPProcessStartupList()
 
 PProcess::PProcess(const char * manuf, const char * name,
                            WORD major, WORD minor, CodeStatus stat, WORD build)
-  : manufacturer(manuf), productName(name)
+  : terminationValue(0)
+  , manufacturer(manuf)
+  , productName(name)
+  , majorVersion(major)
+  , minorVersion(minor)
+  , status(stat)
+  , buildNumber(build)
+  , maxHandles(INT_MAX)
+  , m_library(false)
 {
-  PProcessInstance = this;
-  terminationValue = 0;
+  activeThreads.DisallowDeleteObjects();
+  activeThreads.SetAt((PINDEX)GetCurrentThreadId(), this);
 
-  majorVersion = major;
-  minorVersion = minor;
-  status = stat;
-  buildNumber = build;
+  PProcessInstance = this;
 
 #ifdef P_RTEMS
 
   cout << "Enter program arguments:\n";
   arguments.ReadFrom(cin);
 
-#else // P_RTEMS
-
-  if (p_argv != 0 && p_argc > 0) {
-    executableFile = p_argv[0];
-    arguments.SetArgs(p_argc-1, p_argv+1);
-  }
+#endif // P_RTEMS
 
 #ifdef _WIN32
-  if (executableFile.IsEmpty()) {
-    // Try to get the real image path for this process
-    PVarString moduleName;
-    if (GetModuleFileName(GetModuleHandle(NULL), moduleName.GetPointer(1024), 1024) > 0) {
-      executableFile = moduleName;
-      executableFile.Replace("\\??\\","");
-    }
+  // Try to get the real image path for this process
+  PVarString moduleName;
+  if (GetModuleFileName(GetModuleHandle(NULL), moduleName.GetPointer(1024), 1024) > 0) {
+    executableFile = moduleName;
+    executableFile.Replace("\\??\\","");
   }
-  else {
-    if (!PFile::Exists(executableFile)) {
-      PString execFile = executableFile + ".exe";
-      if (PFile::Exists(execFile))
-        executableFile = execFile;
-    }
-  }
-
-#if defined(P_VIDEO)  && defined(P_VFW_CAPTURE)
-  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
 #endif
 
-#endif // _WIN32
+#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+#  ifdef P_VIDEO
+     PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+#    if defined(P_MINGW) || defined(_WIN32)
+#      ifdef P_VFW_CAPTURE
+         PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#      endif
+#      if defined(P_DIRECTSHOW)
+         PWLibStupidLinkerHacks::loadDirectShowStuff  =1;
+#      endif
+#    endif
+#    ifdef P_FFVDEV
+       PWLibStupidLinkerHacks::loadFFVideoFileStuff = 1;
+#    endif
+#    ifdef P_VIDFILE
+       PWLibStupidLinkerHacks::loadVideoFileStuff = 1;
+#    endif
+#  endif 
+
+#  ifdef P_AUDIO
+#    if defined(P_MINGW) || defined(_WIN32)
+       PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
+#    endif
+#    ifdef P_MACOSX
+       PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
+#    endif
+#  endif
+
+#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
 
   if (productName.IsEmpty())
     productName = executableFile.GetTitle().ToLower();
 
-#endif // P_RTEMS
-
-  InitialiseProcessThread();
-
   Construct();
 
-#if defined(P_MINGW)
-#ifdef P_VIDEO
-  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
-#endif
-#if defined(P_AUDIO)
-  PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
-#endif
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
-#endif
-#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
-  PWLibStupidLinkerHacks::loadDirectShowStuff = 1;
-#endif
-
-#endif
-
-
-#ifdef P_MACOSX
-
-#ifdef P_VIDEO
-
-  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
-  
-#ifdef P_SHM_VIDEO
-  PWLibStupidLinkerHacks::loadShmVideoStuff = 1;
-#endif // P_SHM_VIDEO
-
-#endif
-  
-#ifdef P_AUDIO
-  PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
-#endif // P_AUDIO
-  
-#endif // P_MACOSX
-
   // create one instance of each class registered in the 
   // PProcessStartup abstract factory
   PProcessStartupList & startups = GetPProcessStartupList();
@@ -1617,6 +1645,8 @@ void PProcess::PostShutdown()
     delete startup->second;
 
   startups.clear();
+
+  PProcessInstance = NULL;
 }
 
 
@@ -1643,6 +1673,12 @@ void PProcess::OnThreadEnded(PThread & /*thread*/)
 }
 
 
+bool PProcess::OnInterrupt(bool)
+{
+  return false;
+}
+
+
 PBoolean PProcess::IsInitialised()
 {
   return PProcessInstance != NULL;
@@ -1683,10 +1719,14 @@ PTime PProcess::GetStartTime() const
 
 PString PProcess::GetVersion(PBoolean full) const
 {
-  const char * const statusLetter[NumCodeStatuses] =
-    { "alpha", "beta", "." };
   return psprintf(full ? "%u.%u%s%u" : "%u.%u",
-                  majorVersion, minorVersion, statusLetter[status], buildNumber);
+                  majorVersion, minorVersion, VersionStatus[status], buildNumber);
+}
+
+
+PString PProcess::GetLibVersion()
+{
+  return psprintf("%u.%u%s%u", MAJOR_VERSION, MINOR_VERSION, VersionStatus[BUILD_TYPE], BUILD_NUMBER);
 }
 
 
@@ -1864,6 +1904,24 @@ bool PProcess::HostSystemURLHandlerInfo::Register()
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
 
+PThread * PThread::Current()
+{
+  if (!PProcess::IsInitialised())
+    return NULL;
+
+  PProcess & process = PProcess::Current();
+
+  process.activeThreadMutex.Wait();
+  PThread * thread = process.activeThreads.GetAt((unsigned)GetCurrentThreadId());
+  process.activeThreadMutex.Signal();
+
+  if (thread == NULL)
+    thread = new PExternalThread;
+
+  return thread;
+}
+
+
 void PThread::PrintOn(ostream & strm) const
 {
   strm << GetThreadName();
@@ -2371,24 +2429,4 @@ PWriteWaitAndSignal::~PWriteWaitAndSignal()
   mutex.EndWrite();
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
-PSystemLog::PSystemLog(Level level)   ///< only messages at this level or higher will be logged
-  : iostream(cout.rdbuf()) 
-{ 
-  logLevel = level; 
-  buffer.log = this; 
-  init(&buffer); 
-}
-
-PSystemLog::PSystemLog(const PSystemLog & other)
-  : PObject(other), iostream(cout.rdbuf()) 
-{
-}
-
-PSystemLog & PSystemLog::operator=(const PSystemLog &)
-{ 
-  return *this; 
-}
-
 // End Of File ///////////////////////////////////////////////////////////////
