@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21597 $
+ * $Revision: 22018 $
  * $Author: rjongbloed $
- * $Date: 2008-11-23 12:25:25 +0000 (Sun, 23 Nov 2008) $
+ * $Date: 2009-02-07 02:14:16 +0000 (Sat, 07 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -990,7 +990,12 @@ void SignalHandler(int sig)
   if (PProcess::Current().OnInterrupt(sig == SIGTERM))
     return;
 
-  (sig == SIGTERM ? PreviousSigTermHandler : PreviousSigIntHandler)(sig);
+  void (__cdecl * previous)(int) = (sig == SIGTERM ? PreviousSigTermHandler : PreviousSigIntHandler);
+ 
+  if (previous == SIG_DFL)
+    raise(sig);
+  else if (previous != SIG_IGN)
+    previous(sig);
 }
 #endif
 
